﻿using System;
using System.ComponentModel;
using SpotnetWatchdog.Resources;

namespace SpotnetWatchdog.Entities
{
    class WatchItem : INotifyPropertyChanged, IDataErrorInfo
    {
        #region Private Fields

        private Category _category;
        private string _description;
        private bool _searchForAllWords;
        private string _searchString;
        private string _spotter;
        private SubCategory _subCategory;
        private bool _wholeWordsOnly;

        #endregion

        #region Constructors

        public WatchItem()
        {
            Category = Category.Empty;
            Id = Guid.NewGuid();
            SearchForAllWords = true;
            SubCategory = SubCategory.Empty;
            WholeWordsOnly = false;
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets Category
        /// </summary>
        public Category Category
        {
            get { return _category; }
            set
            {
                if (_category != value)
                {
                    _category = value;
                    RaisePropertyChanged("Category");
                }
            }
        }

        /// <summary>
        /// Gets or sets Description
        /// </summary>
        public string Description
        {
            get { return _description; }
            set
            {
                if (_description != value)
                {
                    _description = value;
                    RaisePropertyChanged("Description");
                }
            }
        }

        public Guid Id { get; private set; }

        /// <summary>
        /// True if the title of a spot should contain all words specified in this.SearchString.
        /// False if the title of a spot should contain atleast one word specified in this.SearchString.
        /// </summary>
        public bool SearchForAllWords
        {
            get { return _searchForAllWords; }
            set
            {
                if (_searchForAllWords != value)
                {
                    _searchForAllWords = value;
                    RaisePropertyChanged("SearchForAllWords");
                }
            }
        }

        /// <summary>
        /// Gets or sets SearchString
        /// </summary>
        public string SearchString
        {
            get { return _searchString; }
            set
            {
                if (_searchString != value)
                {
                    _searchString = value;
                    RaisePropertyChanged("SearchString");

                    SearchStringWords = (_searchString != null ? _searchString.ToLower().Split(' ') : null);
                }
            }
        }

        /// <summary>
        /// Contains all words that are in SearchString
        /// </summary>
        public string[] SearchStringWords { get; private set; }

        /// <summary>
        /// Gets or sets Spotter
        /// </summary>
        public string Spotter
        {
            get { return _spotter; }
            set
            {
                if (_spotter != value)
                {
                    _spotter = value;
                    RaisePropertyChanged("Spotter");
                }
            }
        }

        /// <summary>
        /// Gets or sets SubCategory
        /// </summary>
        public SubCategory SubCategory
        {
            get { return _subCategory; }
            set
            {
                if (_subCategory != value)
                {
                    _subCategory = value ?? SubCategory.Empty;
                    RaisePropertyChanged("SubCategory");
                }
            }
        }

        /// <summary>
        /// Gets or sets WholeWordsOnly
        /// </summary>
        public bool WholeWordsOnly
        {
            get { return _wholeWordsOnly; }
            set
            {
                if (_wholeWordsOnly != value)
                {
                    _wholeWordsOnly = value;
                    RaisePropertyChanged("WholeWordsOnly");
                }
            }
        }
        
        #endregion

        #region Public Methods

        public static WatchItem Copy(WatchItem watchItem)
        {
            if (watchItem == null) return null;

            WatchItem copy = new WatchItem();
            Map(watchItem, copy);
            return copy;
        }

        public static void Map(WatchItem source, WatchItem destination)
        {
            if (source != null && destination != null)
            {
                destination.Category = source.Category;
                destination.Description = source.Description;
                destination.Id = source.Id;
                destination.SearchString = source.SearchString;
                destination.SearchForAllWords = source.SearchForAllWords;
                destination.Spotter = source.Spotter;
                destination.SubCategory = source.SubCategory;
                destination.WholeWordsOnly = source.WholeWordsOnly;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion

        #region IDataErrorInfo Members

        public string Error
        {
            get { throw new NotImplementedException(); }
        }

        public string this[string columnName]
        {
            get 
            {
                return ValidateProperty(columnName);
            }
        }

        private string ValidateProperty(string propertyName)
        {
            if (propertyName == "Description" && string.IsNullOrWhiteSpace(this.Description))
            {
                return Messages.DescriptionIsRequired;
            }
            if (propertyName == "SearchString" && string.IsNullOrWhiteSpace(this.SearchString))
            {
                return Messages.SearchStringIsRequired;
            }
            return null;
        }

        public bool IsValid()
        {
            if (ValidateProperty("Description") != null) return false;
            if (ValidateProperty("SearchString") != null) return false;
            return true;
        }

        #endregion
    }
}
